--[[
Author: Starinnia
CPR is a combo points display addon based on Funkydude's BasicComboPoints
$Date: 2009-01-09 02:31:54 +0000 (Fri, 09 Jan 2009) $
$Revision: 27 $

Copyright (c) 2007-2008 Michael J. Murray aka Starinnia of Lothar

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
]]

local basepath = "Interface\\Addons\\ComboPointsRedux\\Icons\\"
local GetComboPoints = GetComboPoints
local graphics = nil
local text = nil
local defaults = {
	profile = {
		textpos = {},
		graphicpos = {},
		locked = false,
		font = "Friz Quadrata TT",
		graphicscale = 1,
		fontsize = 12,
		spacing = 5,
		normColorr = 1,
		normColorg = 0.9,
		normColorb = 0,
		fullColorr = 1,
		fullColorg = 0,
		fullColorb = 0,
		textColorr = 1,
		textColorg = 1,
		textColorb = 1,
		alpha = 1,
		icon = "square",
		orientation = "h",
	},
}
	
local ComboPointsRedux = LibStub("AceAddon-3.0"):NewAddon("ComboPointsRedux", "AceConsole-3.0", "AceEvent-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ComboPointsRedux")

local cprOps
local function getCPROps()
	if not cprOps then
		cprOps = {
			type = 'group',
			childGroups = 'tab',
			name = "ComboPointsRedux",
			args = {
				main = {
					type = 'group',
					name = L["Options"],
					args = {
						lock = {
							type = "toggle",
							name = L["Lock"],
							get = function() return ComboPointsRedux.db.profile.locked end,
							set = function()
								if not ComboPointsRedux.db.profile.locked then
									graphics:SetBackdropColor(1,1,1,0)
									text:SetBackdropColor(1,1,1,0)
									graphics:EnableMouse(false)
									graphics:SetMovable(false)
									text:EnableMouse(false)
									text:SetMovable(false)
									ComboPointsRedux.db.profile.locked = true
								else
									graphics:SetBackdropColor(1,1,1,1)
									text:SetBackdropColor(1,1,1,1)
									graphics:EnableMouse(true)
									graphics:SetMovable(true)
									text:EnableMouse(true)
									text:SetMovable(true)
									ComboPointsRedux.db.profile.locked = false
								end
							end,
							order = 1,
						},
						orientation = {
							type = 'select',
							name = L["Orientation"],
							get = function() return ComboPointsRedux.db.profile.orientation end,
							set = function(_, value)
								ComboPointsRedux.db.profile.orientation = value
								for i = 1, 5 do
									graphics.points[i]:ClearAllPoints()
								end
								if value == "v" then
									graphics.points[1]:SetPoint("BOTTOM", graphics, "BOTTOM", 0, 0)
									graphics.points[2]:SetPoint("BOTTOM", graphics.points[1], "TOP", 0, ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale)
									graphics.points[3]:SetPoint("BOTTOM", graphics.points[2], "TOP", 0, ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale)
									graphics.points[4]:SetPoint("BOTTOM", graphics.points[3], "TOP", 0, ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale)
									graphics.points[5]:SetPoint("BOTTOM", graphics.points[4], "TOP", 0, ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale)
									graphics:SetHeight(145*ComboPointsRedux.db.profile.graphicscale)
									graphics:SetWidth(25*ComboPointsRedux.db.profile.graphicscale)
								else
									graphics.points[1]:SetPoint("LEFT", graphics, "LEFT", 0, 0)
									graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale, 0)
									graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale, 0)
									graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale, 0)
									graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", ComboPointsRedux.db.profile.spacing*ComboPointsRedux.db.profile.graphicscale, 0)
									graphics:SetWidth(145*ComboPointsRedux.db.profile.graphicscale)
									graphics:SetHeight(25*ComboPointsRedux.db.profile.graphicscale)
								end
							end,
							values = { h = L["Horizontal"], v = L["Vertical"], },
							order = 2,
						},
						font = {
							type = "select",
							name = L["Font"],
							get = function() return ComboPointsRedux.db.profile.font end,
							set = function(_, v)
								ComboPointsRedux.db.profile.font = v
								text.t:SetFont(LSM:Fetch('font', v), ComboPointsRedux.db.profile.fontsize)
							end,
							dialogControl = "LSM30_Font",
							values = AceGUIWidgetLSMlists.font,
							order = 3,
						},
						fontsize = {
							type = "range",
							name = L["Font Size"],
							get = function() return ComboPointsRedux.db.profile.fontsize end,
							set = function(_, v)
								ComboPointsRedux.db.profile.fontsize = v
								text.t:SetFont(LSM:Fetch('font', ComboPointsRedux.db.profile.font), v)
							end,
							min = 1,
							max = 36,
							step = 1,
							width = 'full',
							order = 4,
						},
						graphicscale = {
							type = "range",
							name = L["Graphics Scale"],
							get = function() return ComboPointsRedux.db.profile.graphicscale end,
							set = function(_, scale)
								ComboPointsRedux.db.profile.graphicscale = scale
								for i = 1, 5 do
									graphics.points[i]:SetHeight(25*scale)
									graphics.points[i]:SetWidth(25*scale)
								end
								graphics:SetHeight(25*scale)
								graphics:SetWidth(145*scale)
								graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", ComboPointsRedux.db.profile.spacing*scale, 0)
								graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", ComboPointsRedux.db.profile.spacing*scale, 0)
								graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", ComboPointsRedux.db.profile.spacing*scale, 0)
								graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", ComboPointsRedux.db.profile.spacing*scale, 0)
							end,
							min = .1,
							max = 1.5,
							step = .01,
							width = 'full',
							order = 6,
						},
						spacing = {
							type = 'range',
							name = L["Spacing"],
							get = function() return ComboPointsRedux.db.profile.spacing end,
							set = function(_, v)
								ComboPointsRedux.db.profile.spacing = v
								graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", v*ComboPointsRedux.db.profile.graphicscale, 0)
								graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", v*ComboPointsRedux.db.profile.graphicscale, 0)
								graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", v*ComboPointsRedux.db.profile.graphicscale, 0)
								graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", v*ComboPointsRedux.db.profile.graphicscale, 0)
							end,
							min = -2,
							max = 10,
							step = 1,
							width = 'full',
							order = 7,
						},
						alpha = {
							type = 'range',
							name = L["Alpha"],
							get = function() return ComboPointsRedux.db.profile.alpha end,
							set = function(_, v)
								ComboPointsRedux.db.profile.alpha = v
								text.t:SetAlpha(v)
								text.t:SetShadowColor(0, 0, 0, ComboPointsRedux.db.profile.alpha)
								for i = 1, 5 do
									graphics.points[i]:SetAlpha(v)
								end
							end,
							min = 0,
							max = 1,
							step = .01,
							width = 'full',
							order = 8,
						},
						normalcolor = {
							type = 'color',
							name = L["Normal Color"],
							get = function() return ComboPointsRedux.db.profile.normColorr, ComboPointsRedux.db.profile.normColorg, ComboPointsRedux.db.profile.normColorb end,
							set = function(_, r, g, b)
								ComboPointsRedux.db.profile.normColorr = r
								ComboPointsRedux.db.profile.normColorg = g
								ComboPointsRedux.db.profile.normColorb = b
								ComboPointsRedux:ReDraw()
							end,
							order = 9,
						},
						fullcolor = {
							type = 'color',
							name = L["Five Point Color"],
							get = function() return ComboPointsRedux.db.profile.fullColorr, ComboPointsRedux.db.profile.fullColorg, ComboPointsRedux.db.profile.fullColorb end,
							set = function(_, r, g, b)
								ComboPointsRedux.db.profile.fullColorr = r
								ComboPointsRedux.db.profile.fullColorg = g
								ComboPointsRedux.db.profile.fullColorb = b
								ComboPointsRedux:ReDraw()
							end,
							order = 10,
						},
						textcolor = {
							type = 'color',
							name = L["Text Color"],
							get = function() return ComboPointsRedux.db.profile.textColorr, ComboPointsRedux.db.profile.textColorg, ComboPointsRedux.db.profile.textColorb end,
							set = function(_, r, g, b)
								ComboPointsRedux.db.profile.textColorr = r
								ComboPointsRedux.db.profile.textColorg = g
								ComboPointsRedux.db.profile.textColorb = b
								text.t:SetTextColor(r, g, b)
							end,
							order = 11,
						},
						icon = {
							type = 'select',
							name = L["Combo Point Icon"],
							get = function() return ComboPointsRedux.db.profile.icon end,
							set = function(_, v)
								ComboPointsRedux.db.profile.icon = v
								for i = 1, 5 do
									graphics.points[i].icon:SetTexture(basepath..ComboPointsRedux.db.profile.icon)
								end
							end,
							values = { square = L["Square"], circle = L["Circle"], triangle = L["Triangle"], diamond = L["Diamond"], },
							order = 12,
						},
					},
				},
				profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(ComboPointsRedux.db)
			},
		}
	end
	return cprOps
end

function ComboPointsRedux:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("CPRDB", defaults, "Default")
	
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ComboPointsRedux", getCPROps)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ComboPointsRedux", "ComboPointsRedux")
	self:RegisterChatCommand("cpr", "OpenConfig", true, true)
end

function ComboPointsRedux:OnEnable()
	self:MakeTextFrame()
	self:MakeGraphicsFrame()
	
	self:RegisterEvent("UNIT_COMBO_POINTS", "ReDraw")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "ReDraw")
end

function ComboPointsRedux:Reset()
	text:ClearAllPoints()
	graphics:ClearAllPoints()
	
	text:SetPoint("CENTER", UIParent, "CENTER")
	graphics:SetPoint("CENTER", UIParent, "CENTER")
	
	for i = 1, 5 do
		graphics.points[i].icon:SetTexture(basepath.."square")
	end
	
	graphics:SetWidth(145)
	graphics:SetHeight(25)
end

function ComboPointsRedux:Refresh()
	text:ClearAllPoints()
	graphics:ClearAllPoints()
	
	if self.db.profile.textpos.x then
		text:SetPoint(self.db.profile.textpos.point, UIParent, self.db.profile.textpos.anchor, self.db.profile.textpos.x, self.db.profile.textpos.y)
		graphics:SetPoint(self.db.profile.graphicpos.point, UIParent, self.db.profile.graphicpos.anchor, self.db.profile.graphicpos.x, self.db.profile.graphicpos.y)
	else
		text:SetPoint("CENTER", UIParent, "CENTER")
		graphics:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	for i = 1, 5 do
		graphics.points[i].icon:SetTexture(basepath..self.db.profile.icon)
	end
	
	if self.db.profile.orientation == "v" then
		graphics.points[1]:SetPoint("BOTTOM", graphics, "BOTTOM", 0, 0)
		graphics.points[2]:SetPoint("BOTTOM", graphics.points[1], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
		graphics.points[3]:SetPoint("BOTTOM", graphics.points[2], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
		graphics.points[4]:SetPoint("BOTTOM", graphics.points[3], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
		graphics.points[5]:SetPoint("BOTTOM", graphics.points[4], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
		graphics:SetHeight(145*self.db.profile.graphicscale)
		graphics:SetWidth(25*self.db.profile.graphicscale)
	else
		graphics.points[1]:SetPoint("LEFT", graphics, "LEFT", 0, 0)
		graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
		graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
		graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
		graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
		graphics:SetWidth(145*self.db.profile.graphicscale)
		graphics:SetHeight(25*self.db.profile.graphicscale)
	end
	
	if not self.db.profile.locked then
		text:SetBackdropColor(1,1,1,0)
		text:EnableMouse(false)
		text:SetMovable(false)
		graphics:SetBackdropColor(1,1,1,0)
		graphics:EnableMouse(false)
		graphics:SetMovable(false)
	end
end

function ComboPointsRedux:OpenConfig()
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("ComboPointsRedux", 500, 550)
	LibStub("AceConfigDialog-3.0"):Open("ComboPointsRedux")
end

function ComboPointsRedux:ReDraw()
	--see if the player has a vehicle, if they do get the vehicle's CPs
	local points = GetComboPoints(UnitHasVehicleUI("player") and "vehicle" or "player", "target")
	
	if points == 0 then
		for i = 1, 5 do
			graphics.points[i]:Hide()
			graphics.points[i].icon:SetVertexColor(self.db.profile.normColorr, self.db.profile.normColorg, self.db.profile.normColorb)
		end
		points = ""
	elseif points == 5 then
		for i = points, 1, -1 do
			graphics.points[i].icon:SetVertexColor(self.db.profile.fullColorr, self.db.profile.fullColorg, self.db.profile.fullColorb)
			graphics.points[i]:Show()
		end
	else
		for i = points, 1, -1 do
			graphics.points[i]:Show()
		end
	end
	
	text.t:SetText(points)
end

local bg = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",}
function ComboPointsRedux:MakeTextFrame()
	text = CreateFrame("FRAME", "CPRTEXTFRAME", UIParent)
	text:SetBackdrop(bg)
	text:SetFrameStrata("BACKGROUND")
	text:SetClampedToScreen(true)
	if self.db.profile.textpos.x then
		text:SetPoint(self.db.profile.textpos.point, UIParent, self.db.profile.textpos.anchor, self.db.profile.textpos.x, self.db.profile.textpos.y)
	else
		text:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	text:SetBackdropColor(1,1,1,0)
	text:SetWidth(50)
	text:SetHeight(50)
	
	text.t = text:CreateFontString(nil,"OVERLAY")
	text.t:SetAllPoints(text)
	text.t:SetShadowColor(0, 0, 0, self.db.profile.alpha)
	text.t:SetShadowOffset(1, -1)
	text.t:SetFont(LSM:Fetch("font", self.db.profile.font), self.db.profile.fontsize)
	text.t:SetTextColor(self.db.profile.textColorr, self.db.profile.textColorg, self.db.profile.textColorb)
	text.t:SetAlpha(self.db.profile.alpha)
	
	if not self.db.profile.locked then
		text:SetBackdropColor(1,1,1,1)
		text:EnableMouse(true)
		text:SetMovable(true)
	end
	
	text:RegisterForDrag("LeftButton")
	text:SetScript("OnDragStart", function() this:StartMoving() end)
	text:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local point, _, anchor, x, y = this:GetPoint()
		ComboPointsRedux.db.profile.textpos.x = math.floor(x)
		ComboPointsRedux.db.profile.textpos.y = math.floor(y)
		ComboPointsRedux.db.profile.textpos.anchor = anchor
		ComboPointsRedux.db.profile.textpos.point = point
	end)
	
	text:Show()
end

function ComboPointsRedux:MakeGraphicsFrame()
	graphics = CreateFrame("FRAME", "CPRGRAPHICSFRAME", UIParent)
	graphics:SetBackdrop(bg)
	graphics:SetFrameStrata("BACKGROUND")
	graphics:SetClampedToScreen(true)
	if self.db.profile.graphicpos.x then
		graphics:SetPoint(self.db.profile.graphicpos.point, UIParent, self.db.profile.graphicpos.anchor, self.db.profile.graphicpos.x, self.db.profile.graphicpos.y)
	else
		graphics:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	graphics:SetBackdropColor(1,1,1,0)
	if self.db.profile.orientation == "v" then
		graphics:SetHeight(145*self.db.profile.graphicscale)
		graphics:SetWidth(25*self.db.profile.graphicscale)
	else
		graphics:SetWidth(145*self.db.profile.graphicscale)
		graphics:SetHeight(25*self.db.profile.graphicscale)
	end
	
	if not self.db.profile.locked then
		graphics:SetBackdropColor(1,1,1,1)
		graphics:EnableMouse(true)
		graphics:SetMovable(true)
	end
	
	graphics:RegisterForDrag("LeftButton")
	graphics:SetScript("OnDragStart", function() this:StartMoving() end)
	graphics:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local point, _, anchor, x, y = this:GetPoint()
		ComboPointsRedux.db.profile.graphicpos.x = math.floor(x)
		ComboPointsRedux.db.profile.graphicpos.y = math.floor(y)
		ComboPointsRedux.db.profile.graphicpos.anchor = anchor
		ComboPointsRedux.db.profile.graphicpos.point = point
	end)
	
	graphics.points = {}
	for i = 1, 5 do
		graphics.points[i] = CreateFrame("FRAME", nil, graphics)
		graphics.points[i].icon = graphics.points[i]:CreateTexture(nil, "OVERLAY")
		graphics.points[i].icon:SetAllPoints(graphics.points[i])
		graphics.points[i].icon:SetTexture(basepath..self.db.profile.icon)
		graphics.points[i].icon:SetVertexColor(self.db.profile.normColorr, self.db.profile.normColorg, self.db.profile.normColorb)
		graphics.points[i]:SetAlpha(self.db.profile.alpha)
		graphics.points[i]:SetHeight(25*self.db.profile.graphicscale)
		graphics.points[i]:SetWidth(25*self.db.profile.graphicscale)
		graphics.points[i]:Hide()
	end
	
	if self.db.profile.orientation == "v" then
		graphics.points[1]:SetPoint("BOTTOM", graphics, "BOTTOM", 0, 0)
		graphics.points[2]:SetPoint("BOTTOM", graphics.points[1], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
		graphics.points[3]:SetPoint("BOTTOM", graphics.points[2], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
		graphics.points[4]:SetPoint("BOTTOM", graphics.points[3], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
		graphics.points[5]:SetPoint("BOTTOM", graphics.points[4], "TOP", 0, self.db.profile.spacing*self.db.profile.graphicscale)
	else
		graphics.points[1]:SetPoint("LEFT", graphics, "LEFT", 0, 0)
		graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
		graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
		graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
		graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", self.db.profile.spacing*self.db.profile.graphicscale, 0)
	end
	
	graphics:Show()
end
